#include "joystick.h"

#define WAITFAM      Delay10TCYx(5)   // 6us
#define WAITFAM_HALF Delay10TCYx(2)
#define FC_LATCH (PORTCbits.RC0)
#define FC_DATA  (PORTCbits.RC1)
#define FC_CLOCK (PORTCbits.RC2)
#define FC_LATCH_L_CLOCK_H  LATC = 0x04
#define FC_LATCH_H_CLOCK_H  LATC = 0x05
#define FC_LATCH_L_CLOCK_L  LATC = 0x00


// Prottype@definition
void PrepareForReadFC(void);

/// -------------------------------------------------------
/// ǂݎ菀i|[gݒȂǁj
/// -------------------------------------------------------
void PrepareForReadFC(void)
{
    // port IN/OUT setup for Famicom
    TRISC = 0b11111010;

    // Prepare for read
    FC_LATCH_L_CLOCK_H;
    WAIT_PORTSET;
}



void ReadPhaseDataFC(unsigned char *phaseData)
{
    // LATCH H->L
    FC_LATCH_H_CLOCK_H;
    WAITFAM;
    WAITFAM;
    FC_LATCH_L_CLOCK_H;
    WAITFAM;

    for(int i=0; i<24; i=i+1)
    {
        //FC_CLOCK = 0;  // CLOCK = L
        FC_LATCH_L_CLOCK_L;
        WAITFAM;
        phaseData[i] = FC_DATA;

        //FC_CLOCK = 1;  // CLOCK = H
        FC_LATCH_L_CLOCK_H;
        WAITFAM;
    }
}


/// -------------------------------------------------------
/// ǂݎ聕WCXeBbNo̓f[^쐬
/// iWCJ[hmkII،Rg[[ɏRg[[j
/// -------------------------------------------------------
bool ReadFamicomJoycard
    (char *devType, int *buttons)
{
    unsigned char phaseData[24];
    static unsigned int counter_Failed = 0;
    bool isFamicom = false;

    if(*devType == DEVTYPE_UNKNOWN)
    {
        PrepareForReadFC();
    }


    // If DATA==L when LATCH=L, Famicom Controller connected.
    // E\Bł͌młȂ͂
    // A{^B{^ƌmł͂߂
    // uA{^B{^񂩉ƃfoCXF܂vƂ
    if(FC_DATA == 0)
    {
        isFamicom = true;
    }

    // Read
    ReadPhaseDataFC(phaseData);


    // Connection detection
    if(isFamicom)
    {
        counter_Failed = 0;

        *devType = DEVTYPE_FAMICOM;

        //Buttons
#ifdef Output_PS3
        // vXe3VԂ̂ɕ֗ȃL[蓖
        *buttons = 0x00000000 |
            ((phaseData[ 0]) ? 0 : BTNBIT_02) |  // FC A, SFC B
            ((phaseData[ 1]) ? 0 : BTNBIT_01) |  // FC B, SFC Y
            ((phaseData[ 2]) ? 0 : BTNBIT_09) |  // Select
            ((phaseData[ 3]) ? 0 : BTNBIT_10) ;  // Start
        /*
         * SFCRg[،Rg[͈ȉ̃L[gp\B
         * AFC̃WCJ[hmkIIł͉ςȂɂȂĂ܂B
            ((phaseData[ 8]) ? 0 : BTNBIT_03) |  // (SFC A,  ، 0 -> 03)
            ((phaseData[ 9]) ? 0 : BTNBIT_04) |  // (SFC X,  ، 1 -> 04)
            ((phaseData[10]) ? 0 : BTNBIT_05) |  // (SFC Lt, ، 2 -> 05)
            ((phaseData[11]) ? 0 : BTNBIT_06) |  // (SFC Rt, ، 3 -> 06)
            ((phaseData[12]) ? 0 : BTNBIT_07) |  // (        ، 4 -> 07)
            ((phaseData[13]) ? 0 : BTNBIT_08) |  // (        ، 5 -> 08)
            ((phaseData[14]) ? 0 : BTNBIT_11) |  // (        ، 6 -> 11)
            ((phaseData[15]) ? 0 : BTNBIT_12) |  // (        ، 7 -> 12)
            ((phaseData[16]) ? 0 : BTNBIT_13) |  // (        ، 8 -> 13) PS3 Home
            ((phaseData[17]) ? 0 : BTNBIT_14) |  // (        ، 9 -> 14)
            ((phaseData[18]) ? 0 : BTNBIT_15) |  // (        ، * -> 15)
            ((phaseData[19]) ? 0 : BTNBIT_16) ;  // (        ، # -> 16)
        */

        // Home
        if(
            // A+B+Sel+Start-> Home
            (*buttons & BTNBIT_01) &&
            (*buttons & BTNBIT_02) &&
            (*buttons & BTNBIT_09) &&
            (*buttons & BTNBIT_10)
        )
        {
            *buttons = (*buttons | BTNBIT_13) &
            (~(BTNBIT_01 |
               BTNBIT_02 |
               BTNBIT_09 |
               BTNBIT_10 ));
        }
#else
        // Nintendo SwitchVԂ̂ɕ֗ȃL[蓖
        *buttons = 0x00000000 |
            ((phaseData[ 0]) ? 0 : BTNBIT_NSW_A)     |  // FC A, SFC B -> NSW A
            ((phaseData[ 1]) ? 0 : BTNBIT_NSW_B)     |  // FC B, SFC Y -> NSW_B
            ((phaseData[ 2]) ? 0 : BTNBIT_NSW_MINUS) |  // Select      -> NSW -
            ((phaseData[ 3]) ? 0 : BTNBIT_NSW_PLUS)  ;  // Start       -> NSW +

        // ꑀ
        if(
            // Sel+Start-> Home
            (*buttons & BTNBIT_NSW_MINUS) &&
            (*buttons & BTNBIT_NSW_PLUS)
        )
        {
            *buttons = (*buttons | BTNBIT_NSW_HOME) &
            (~(BTNBIT_NSW_MINUS |
               BTNBIT_NSW_PLUS ));
        }

        if(
            // Sel+A-> X
            (*buttons & BTNBIT_NSW_MINUS) &&
            (*buttons & BTNBIT_NSW_A)
        )
        {
            *buttons = (*buttons | BTNBIT_NSW_X) &
            (~(BTNBIT_NSW_MINUS |
               BTNBIT_NSW_A ));
        }

        if(
            // Sel+B-> Y
            (*buttons & BTNBIT_NSW_MINUS) &&
            (*buttons & BTNBIT_NSW_B)
        )
        {
            *buttons = (*buttons | BTNBIT_NSW_Y) &
            (~(BTNBIT_NSW_MINUS |
               BTNBIT_NSW_B ));
        }

        if(
            // Sel+Left-> Lt
            (*buttons & BTNBIT_NSW_MINUS) &&
            (phaseData[6] == 0)
        )
        {
            *buttons = (*buttons | BTNBIT_NSW_LT) &
            (~(BTNBIT_NSW_MINUS));
        }

        if(
            // Sel+Right-> Rt
            (*buttons & BTNBIT_NSW_MINUS) &&
            (phaseData[7] == 0)
        )
        {
            *buttons = (*buttons | BTNBIT_NSW_RT) &
            (~(BTNBIT_NSW_MINUS));
        }

        if(
            // Sel+Up-> ZL
            (*buttons & BTNBIT_NSW_MINUS) &&
            (phaseData[4] == 0)
        )
        {
            *buttons = (*buttons | BTNBIT_NSW_ZL) &
            (~(BTNBIT_NSW_MINUS));
        }

        if(
            // Sel+Down-> ZR
            (*buttons & BTNBIT_NSW_MINUS) &&
            (phaseData[5] == 0)
        )
        {
            *buttons = (*buttons | BTNBIT_NSW_ZR) &
            (~(BTNBIT_NSW_MINUS));
        }

#endif

        Set_AnalogAndHatValue(
            (phaseData[7]),     // R
            (phaseData[6]),     // L
            (phaseData[4]),     // U
            (phaseData[5])      // D
            );

    }
    else
    {
        counter_Failed++;
        if(counter_Failed > 10)
        {
            counter_Failed = 0;
            *devType = DEVTYPE_UNKNOWN;
        }
    }

    return isFamicom;

}