#include "joystick.h"

#define WAIT_SS    Delay10TCYx(20)   // 24us
#ifdef  PROTOTYPE_1
 // for Prototype Box
 #define SS_INIT_TRISC      0b11001111
 #define SS_TL              PORTCbits.RC4
 #define SS_TR              PORTCbits.RC5
 #define SS_SEND_TL_H_TR_H  LATC = 0x30
 #define SS_SEND_TL_L_TR_H  LATC = 0x20
 #define SS_SEND_TL_H_TR_L  LATC = 0x10
 #define SS_SEND_TL_L_TR_L  LATC = 0x00
#else
 // for Version 1.1 circuit
 #define SS_INIT_TRISC      0b10101111
 #define SS_TL              PORTCbits.RC4
 #define SS_TR              PORTCbits.RC6
 #define SS_SEND_TL_H_TR_H  LATC = 0x50
 #define SS_SEND_TL_L_TR_H  LATC = 0x40
 #define SS_SEND_TL_H_TR_L  LATC = 0x10
 #define SS_SEND_TL_L_TR_L  LATC = 0x00
#endif
#define SS_DATA        PORTC
#define SS_PAD_NONE    0
#define SS_PAD_NORMAL  1
#define SS_PAD_MUL_A   2
#define SS_PAD_MUL_D   3
#define SS_MUL_A_POLLING_WAIT   Delay1KTCYx(24)  // Web site says 1ms
#define SS_MUL_D_POLLING_WAIT   Delay1KTCYx(24)  // 83usec x multiplier
#define SS_NORM_POLLING_WAIT    Delay1KTCYx(12)

// Prototype definition
void PrepareForReadSS(void);

/// -------------------------------------------------------
/// ǂݎ菀i|[gݒȂǁj
/// -------------------------------------------------------
void PrepareForReadSS(void)
{
    // Initialize port IN/OUT direction for SATURN
    TRISC = SS_INIT_TRISC;

    // Prepare for read
    SS_SEND_TL_H_TR_H;
    WAIT_PORTSET;
}

void ReadPhaseDataSSNormal(unsigned char *phaseData)
{
    // Read data 3 phase
    SS_SEND_TL_H_TR_L;
    WAIT_SS;
    phaseData[1] = SS_DATA & 0x0f;

    SS_SEND_TL_L_TR_H;
    WAIT_SS;
    phaseData[2] = SS_DATA & 0x0f;

    SS_SEND_TL_L_TR_L;
    WAIT_SS;
    phaseData[3] = SS_DATA & 0x0f;

    // Read done. Set pin status for next read
    SS_SEND_TL_H_TR_H;
}

void ReadPhaseDataSSMul(unsigned char *phaseData)
{
    SS_SEND_TL_L_TR_H;

    for(int i=0; i<16; i++)
    {
        if( (i+1) & 0x01 )
        {
            SS_SEND_TL_L_TR_H;
        }
        else
        {
            SS_SEND_TL_L_TR_L;
        }

        WAIT_SS;
        phaseData[i] = SS_DATA & 0x0f;
    }
    SS_SEND_TL_H_TR_H;
}

/// -------------------------------------------------------
/// ǂݎ聕WCXeBbNo̓f[^쐬
/// iZKT^[WpbhA}RD/Aj
/// -------------------------------------------------------
bool ReadSS
    (char *devType, int *buttons)
{
    G_USBINT = false;

    unsigned char phaseData[16];
    // 1: Normal
    // 2: Mulcon(A)
    // 3: Mulcon(D)
    bool state = true;


    static char padType = 0;
    static char newPadType = 0;

    if(*devType == DEVTYPE_UNKNOWN)
    {
        PrepareForReadSS();
    }

    // Read first data
    phaseData[0] = SS_DATA & 0x0f;

    if((phaseData[0] & 0x07) == 0x04)
    {
        // look like Normal Pad
        *devType = DEVTYPE_SS;
        newPadType = SS_PAD_NORMAL;

        ReadPhaseDataSSNormal(phaseData);
        /*
        // Read data 3 phase
        SS_SEND_TL_H_TR_L;
        WAIT_SS;
        phaseData[1] = SS_DATA & 0x0f;

        SS_SEND_TL_L_TR_H;
        WAIT_SS;
        phaseData[2] = SS_DATA & 0x0f;

        SS_SEND_TL_L_TR_L;
        WAIT_SS;
        phaseData[3] = SS_DATA & 0x0f;

        // Read done. Set pin status for next read
        SS_SEND_TL_H_TR_H;
        */

        state = true;
    }
    else
    {
        // Mulcon(D mode/Amode) or not connected
        ReadPhaseDataSSMul(phaseData);
        /*
        SS_SEND_TL_L_TR_H;

        for(int i=0; i<16; i++)
        {
            if( (i+1) & 0x01 )
            {
                SS_SEND_TL_L_TR_H;
            }
            else
            {
                SS_SEND_TL_L_TR_L;
            }

            WAIT_SS;
            phaseData[i] = SS_DATA & 0x0f;
        }
        SS_SEND_TL_H_TR_H;
        */

        // Device detection
        if(
            (phaseData[0]==0x01) && (phaseData[1]==0x01) &&
            (phaseData[2]==0x06) )
        {
            // Mulcon A mode
            *devType = DEVTYPE_SS;
            newPadType = SS_PAD_MUL_A;
            state = true;
        }
        else if(
            (phaseData[0]==0x01) && (phaseData[1]==0x00) &&
            (phaseData[2]==0x02) )
        {
            // Mulcon D mode
            *devType = DEVTYPE_SS;
            newPadType = SS_PAD_MUL_D;
            state = true;
        }
        else
        {
            *devType = DEVTYPE_UNKNOWN;
            newPadType = SS_PAD_NONE;
            state = false;
        }
    }


    if(state && (G_USBINT==false))
    {
        //*devType = newDevType;
        padType = newPadType;

        switch(padType)
        {
            case SS_PAD_NORMAL:
                //Buttons
                *buttons = 0x00000000 |
                    ((phaseData[1]&0b0100) ? 0 : BTNBIT_01) |  // A
                    ((phaseData[1]&0b0001) ? 0 : BTNBIT_02) |  // B
                    ((phaseData[1]&0b0010) ? 0 : BTNBIT_03) |  // C
                    ((phaseData[3]&0b0100) ? 0 : BTNBIT_04) |  // X
                    ((phaseData[3]&0b0010) ? 0 : BTNBIT_05) |  // Y
                    ((phaseData[3]&0b0001) ? 0 : BTNBIT_06) |  // Z
                    ((phaseData[0]&0b1000) ? 0 : BTNBIT_07) |  // L
                    ((phaseData[3]&0b1000) ? 0 : BTNBIT_09) |  // R
                    ((phaseData[1]&0b1000) ? 0 : BTNBIT_10) ;  // Start


                // Home
                if(
                    // A+B+C+Ltrigger+Start -> Home
                    (*buttons & BTNBIT_01) &&
                    (*buttons & BTNBIT_02) &&
                    (*buttons & BTNBIT_03) &&
                    (*buttons & BTNBIT_07) &&
                    (*buttons & BTNBIT_10)
                )
                {
                    *buttons = (*buttons | BTNBIT_13) &
                    (~(BTNBIT_01 |
                       BTNBIT_02 |
                       BTNBIT_03 |
                       BTNBIT_07 |
                       BTNBIT_10 ));
                }

                Set_AnalogAndHatValue(
                    (phaseData[2]&0b1000),
                    (phaseData[2]&0b0100),
                    (phaseData[2]&0b0001),
                    (phaseData[2]&0b0010)
                    /*,hat, x, y, z, r*/);

                SS_NORM_POLLING_WAIT;

                break;

            case SS_PAD_MUL_A:
                //Buttons
                *buttons = 0x00000000 |
                    ((phaseData[4]&0b0100) ? 0 : BTNBIT_01) |  // A
                    ((phaseData[4]&0b0001) ? 0 : BTNBIT_02) |  // B
                    ((phaseData[4]&0b0010) ? 0 : BTNBIT_03) |  // C
                    ((phaseData[5]&0b0100) ? 0 : BTNBIT_04) |  // X
                    ((phaseData[5]&0b0010) ? 0 : BTNBIT_05) |  // Y
                    ((phaseData[5]&0b0001) ? 0 : BTNBIT_06) |  // Z
                    ((phaseData[6]&0b1000) ? 0 : BTNBIT_07) |  // L
                    ((phaseData[5]&0b1000) ? 0 : BTNBIT_08) |  // R
                    ((phaseData[4]&0b1000) ? 0 : BTNBIT_10) ;  // Start

                // Home
                if(
                    // A+B+C+Ltrigger+Start -> Home
                    (*buttons & BTNBIT_01) &&
                    (*buttons & BTNBIT_02) &&
                    (*buttons & BTNBIT_03) &&
                    (*buttons & BTNBIT_07) &&
                    (*buttons & BTNBIT_10)
                )
                {
                    *buttons = (*buttons | BTNBIT_13) &
                    (~(BTNBIT_01 |
                       BTNBIT_02 |
                       BTNBIT_03 |
                       BTNBIT_07 |
                       BTNBIT_10 ));
                }

                //Hat switch
                joystick_input.val[2] =
                    Get_HatSwitchVal(
                        (phaseData[3]&0b1000),
                        (phaseData[3]&0b0100),
                        (phaseData[3]&0b0001),
                        (phaseData[3]&0b0010) );

                //Analog sticks
                joystick_input.val[3] = (phaseData[ 7]<<4) | phaseData[ 8];
                joystick_input.val[4] = (phaseData[ 9]<<4) | phaseData[10];
                joystick_input.val[5] = (phaseData[11]<<4) | phaseData[12];
                joystick_input.val[6] = (phaseData[13]<<4) | phaseData[14];

                SS_MUL_A_POLLING_WAIT;

                break;

            case SS_PAD_MUL_D:
                //Buttons
                *buttons = 0x00000000 |
                    ((phaseData[4]&0b0100) ? 0 : BTNBIT_01) |  // A
                    ((phaseData[4]&0b0001) ? 0 : BTNBIT_02) |  // B
                    ((phaseData[4]&0b0010) ? 0 : BTNBIT_03) |  // C
                    ((phaseData[5]&0b0100) ? 0 : BTNBIT_04) |  // X
                    ((phaseData[5]&0b0010) ? 0 : BTNBIT_05) |  // Y
                    ((phaseData[5]&0b0001) ? 0 : BTNBIT_06) |  // Z
                    ((phaseData[6]&0b1000) ? 0 : BTNBIT_07) |  // L
                    ((phaseData[5]&0b1000) ? 0 : BTNBIT_08) |  // R
                    ((phaseData[4]&0b1000) ? 0 : BTNBIT_10) ;  // Start

                // Home
                if(
                    // A+B+C+Ltrigger+Start -> Home
                    (*buttons & BTNBIT_01) &&
                    (*buttons & BTNBIT_02) &&
                    (*buttons & BTNBIT_03) &&
                    (*buttons & BTNBIT_07) &&
                    (*buttons & BTNBIT_10)
                )
                {
                    *buttons = (*buttons | BTNBIT_13) &
                    (~(BTNBIT_01 |
                       BTNBIT_02 |
                       BTNBIT_03 |
                       BTNBIT_07 |
                       BTNBIT_10 ));
                }

                Set_AnalogAndHatValue(
                    (phaseData[3]&0b1000),
                    (phaseData[3]&0b0100),
                    (phaseData[3]&0b0001),
                    (phaseData[3]&0b0010)/*,
                    hat, x, y, z, r*/);

                SS_MUL_D_POLLING_WAIT;

            default:
                break;
        }
    }

    return true;
}
